"""Include functions to work with markdown text.

License:
    MIT License
    Copyright (c) 2024 François Pelletier
"""

import re


def replace_title_with_list(text: str) -> str:
    r"""Replace all titles in the text with a numbered list.

    The titles are defined by a hashtag symbol followed by one or more
    hashtag symbols, followed by a space, and then the title text.
    The function replaces the entire hashtag symbol and space with a
    numbered list, where each item in the list is numbered with a
    hashtag symbol and a space, and the title text is listed after the
    number.

    Args:
        text (str): The text to process.

    Returns:
        str: The processed text.
    ---

    Examples:
        >>> replace_title_with_list("# This is a title")
        '- This is a title'
        >>> replace_title_with_list("# This is a title\n## Another title")
        '- This is a title\n  - Another title'
    ---

    """
    pattern = r'^#+'
    matches = re.findall(pattern, text, re.MULTILINE)
    for match in matches:
        nb_hashtag = len(match)
        if nb_hashtag > 1:
            text = re.sub(match, '  ' * (nb_hashtag-1)+'-', text, count=1)
    text = re.sub('\n\n', '\n', text)
    pattern_url = \
        r"(?i)\b((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'\".,<>?«»“”‘’]))"
    matches_url = re.findall(pattern_url, text, re.MULTILINE)
    for match in matches_url:
        text = text.replace(str(match[0]), f'[{str(match[0])}]({str(match[0])})')
    return text
